/*
 * Choice JS
*/

class Builder {

    //Constructor
    constructor(target, userOptions){
        this.target = target;
        //Build our default options
        this.options = {
            ...this.buildDefaultOptions(userOptions),
            ...userOptions
        };
    }
    
    /**
     * Create a new choice dropdown
     * 
     * @return {Promise}
    */
    async create(){
        //Build it
        let choice = await this.build();
        //Toast bindings
        await this.bindings(choice);
        //Configure the select based on the options
        await this.configureOptions(choice);
        //Hide the current
        this.target.classList.add('choicejs-hidden-select');
        //Deploy the select
        this.target.parentNode.insertBefore(choice, this.target.nextSibling);
        //Done
        return choice;
    }
    
    
    /**
     * Build the choice
     * 
     * @return {Promise} HTMLElement The new choice
    */
    async build(){
        //Build the new Select HTML element and add it directly after the target
        let wrapper = document.createElement('div'),
            select = document.createElement('div'),
            search = document.createElement('input'),
            dropdown = document.createElement('div'),
            list = document.createElement('ul');
        //Set attributes
        wrapper.classList.add('choicejs');
        select.classList.add('choicejs-select');
        search.classList.add('choicejs-search');
        search.setAttribute('placeholder', 'Search');
        dropdown.classList.add('choicejs-dropdown');
        //Loop the targets options and build our list
        for(let option of this.target.querySelectorAll('option')){
            if(option.value != '' && option.value != undefined){    
                let item = document.createElement('li');
                item.setAttribute('value', option.value);
                item.innerHTML = option.innerHTML;
                list.appendChild(item);
            }
        }
        //Build the whole select and return it
        dropdown.appendChild(search);
        dropdown.appendChild(list);
        wrapper.appendChild(select);
        wrapper.appendChild(dropdown);
        //Return
        return wrapper;
    }
    

    /**
     * Configure options for the choice
     * 
     * @param {HTMLElement} choice - The select we're configuring options for
     * @return {Promise}
    */
    async configureOptions(choice){
        
        //Width and height
        choice.style.width = (this.options.width == 'inherit') ? (this.target.offsetWidth + 'px') : this.options.width,
        choice.style.minHeight = (this.options.height == 'inherit') ? (this.target.offsetHeight + 'px') : this.options.height;
        
        //Class and id
        if(this.options.id != ''){
            choice.setAttribute('id', this.options.id);
        }
        if(this.options.class != ''){
            choice.classList.add(this.options.class);
        }
        
        //Return
        return;
    }
    
    
    /**
     * Build the default options
     * 
     * @param {Object} userOptions The user options for any overwrites
     * @return {Object} An object of options
    */
    buildDefaultOptions(userOptions){
        let options = {
            width:'inherit',
            height:'inherit',
            placeholder: (this.target.hasAttribute('placeholder') ? this.target.getAttribute('placeholder') : 'Select...'),
            id: '',
            class: '',
            closeOnSelect: false,
            event: 'change'
        }
        //Return
        return options;
    }
    
    
    /**
     * Add all of the bindings for the choice
     * 
     * @param {HTMLElement} choice - The select we're building the bindings for
     * @return {Promise}
    */
    async bindings(choice){
        //Return
        return await Promise.all([
            this.bindActiveListener(choice),
            this.bindOptionFiltering(choice),
            this.bindOptionSelect(choice),
        ]);
    }
    
    
    /**
     * Bind active click
     *
     * @param {HTMLElement} choice - The choice we're binding
     * @return {Promise}
    */
    async bindActiveListener(choice){
        choice.addEventListener('click', () => {
            choice.classList.add('active');
        })
        document.addEventListener('click', function(event) {
            if (!choice.contains(event.target)) {
                choice.classList.remove('active');
            }
        });
        //Return
        return;
    }
    
    
    
    /**
     * Bind option select
     * 
     * @param {HTMLElement} choice - The choice we're binding
     * @return {Promise}
    */
    async bindOptionSelect(choice){
        //Single or multiple select?
        let multiple = this.target.getAttribute('multiple'),
            bindings = [];
        //Loop options and bind
        for(let option of choice.querySelectorAll('ul li')){
            if(multiple && multiple == 'true'){
                bindings.push(this.bindMultipleOptionSelect(option));
            } else {
                bindings.push(this.bindSingleOptionSelect(option));
            }
        }   
        //Return
        return Promise.all(bindings);
    }
    
    
    /**
     * Single option click
     *
     * @param {HTMLElement} option - The option
     * @returns {Promise}
    */
    async bindSingleOptionSelect(option){
        //Elems
        let choice = option.closest('.choicejs'),
            select = choice.querySelector('.choicejs-select');
        //Add event listener
        option.addEventListener('click', async (e) => {
            e.stopPropagation();
            e.stopImmediatePropagation();
            //Get the value we've selected
            let value = option.getAttribute('value');
            //Take off all siblings
            for(let sibling of choice.querySelectorAll('li')){
                sibling.classList.remove('selected');
            }
            //Toggle the selected class
            option.classList.toggle('selected');
            //Set the value
            select.innerHTML = option.innerHTML;
            for(let hiddenOption of this.target.querySelectorAll('option')){
                hiddenOption.removeAttribute('selected');
            }
            this.target.querySelector(`option[value="${value}"]`).setAttribute('selected', true);
            //Close on select?
            if(this.options.closeOnSelect){
                choice.classList.remove('active');
            }
            //What parent event to trigger?
            if(this.options.event !== false){
                this.target.dispatchEvent(new Event(this.options.event));
            }
        })
        //Return
        return;
    }
    
    
    /**
     * Multiple option click
     *
     * @param {HTMLElement} option - The option
     * @returns {Promise}
    */
    async bindMultipleOptionSelect(option){
        //Elems
        let choice = option.closest('.choicejs'),
            select = choice.querySelector('.choicejs-select');
        //Add event listener
        option.addEventListener('click', async (e) => {
            e.stopPropagation();
            e.stopImmediatePropagation();
            //Get the value we've selected
            let value = option.getAttribute('value');
            //First check if the option is already selected
            if(option.classList.contains('selected')){
                //Remove
                this.target.querySelector(`option[value="${value}"]`).removeAttribute('selected');
                select.querySelector(`.choicejs-pill[value="${value}"]`).remove();
            } else {
                //Select it
                this.target.querySelector(`option[value="${value}"]`).setAttribute('selected', true);
                let pill = await this.buildOptionPill(option);
                //Bind the remove button
                await this.bindPillRemoveBtn(pill, choice);
                //Append it
                select.appendChild(pill);
            }
            //Toggle the selected class
            option.classList.toggle('selected');
            //Close on select?
            if(this.options.closeOnSelect){
                choice.classList.remove('active');
            }
            //What parent event to trigger?
            if(this.options.event !== false){
                this.target.dispatchEvent(new Event(this.options.event));
            }
        })
        //Return
        return;
    }
    
    
    /**
     * Build option pill
     *
     * @param {Node} option - The option selected
     * @returns {Promise} A node element
    */
    async buildOptionPill(option){
        let wrapper = document.createElement('div'),
            close = document.createElement('span'),
            value = option.getAttribute('value');
        //Attributes
        wrapper.classList.add('choicejs-pill');
        wrapper.setAttribute('value', value);
        wrapper.innerHTML = option.innerHTML;
        //Build it
        wrapper.appendChild(close);
        //Return
        return wrapper;
    }
    
    
    
    /**
     * Bind option filtering
     * 
     * @param {HTMLElement} choice - The choice we're binding
     * @return {Promise}
    */
    async bindOptionFiltering(choice){
        //When the choice search bar changes
        var searchTimeout = 0,
            searchBar = choice.querySelector('.choicejs-search'),
            options = choice.querySelectorAll('ul li');
        //When the postcode is changed, test it
        searchBar.addEventListener('input', () => {
            //Clear the above timeout
            clearTimeout(searchTimeout);
            //Set a new timeout
            searchTimeout = setTimeout(() =>{
                //Loop all options
                for(let option of options){
                    if(option.innerHTML.includes(searchBar.value)){
                        option.classList.remove('hide');
                    } else {
                        option.classList.add('hide');
                    }
                }
            }, 500);
        });
        //Return
        return;
    }
    
    
    /**
     * Bind pill remove btn
     *
     * @param {Node} pill - The pill
     * @param {HTMLElement} choice - The choice instance
     * @returns {Promise}
    */
    async bindPillRemoveBtn(pill, choice){
        //Params
        let value = pill.getAttribute('value');
        //Add the remove event listener
        pill.querySelector('span').addEventListener('click', (e) => {
            e.stopPropagation();
            e.stopImmediatePropagation();
            this.target.querySelector(`option[value="${value}"]`).removeAttribute('selected');
            choice.querySelector(`.choicejs-dropdown li[value="${value}"]`).classList.remove('selected');
            pill.remove();
        })
        //Return
        return;
    }

}





/**
 * Export the choice function to use
 *
 * @param {NodeElement} target - The target we're adding the choice to
 * @param {Object} options - The options
 * @returns {Promise}  - An instance of choice
 */
export const Choice = async (target, options) => {
    //Start a new Choice
    let newBuild = new Builder(target, options);
    //Run the build
    return await newBuild.create();
}